"""Module containing the plotting classes. The methods correspond to different types of plots to create"""

try:
    import numpy as np
    import matplotlib.pyplot as plt
    import seaborn as sns
    import pandas as pd
    from natsort import natsorted
    from bokeh.plotting import figure, output_file, show
    from bokeh.models import Whisker, BasicTicker, ColorBar, LinearColorMapper, PrintfTickFormatter
    import colorcet as cc
    import bokeh as bk
    import math
except ModuleNotFoundError:
    raise ModuleNotFoundError('Some dependencies might be missing. Check installation and try again')
except Exception as err:
    print('Unexpected Error')
    print(err)
else:
    if __name__ == '__main__':
        print("Modules have been loaded")


class Plot:
    """
    Plot objects Master Class from which the rest inherit

    :param stack: Value to denote if barplots should stack
    :type stack: Bool
    :param value: Data to be plotted. Can be 'isotopologue_fraction', 'corrected area' or 'mean_enrichment'
    :type value: str
    :param data: IsoplotData object containing clean data
    :type data: Pandas Dataframe
    :param name: Name for generated file directory where plots will go
    :type name: str
    :param metabolite: metabolite to be plotted
    :type metabolite: str
    :param condition: List of conditions to be plotted
    :type condition: list
    :param time: List of times to be plotted
    :type time: list
    """

    WIDTH = 1080
    HEIGHT = 640

    def __init__(self, stack, value, data, name, metabolite, condition, time):

        self.stack = stack
        self.value = value
        self.data = data
        self.name = name
        self.metabolite = metabolite
        self.condition = condition
        self.time = time
        self.filtered_data = self.data[
            (self.data['metabolite'] == self.metabolite) &
            (self.data['condition'].isin(self.condition)) &
            (self.data['time'].isin(self.time))]

    @staticmethod
    def split_ids(ids):
        """
        Function to split IDs and get back lists of conditions, times and replicates in order

        :param ids: IDs generated by IsoplotData Object
        :type ids: list
        :return: lists containing conditions, times and replicates
        :rtype: lists
        """

        count = ids[0].count('_')

        if count == 2:

            conditions, times, replicates = [], [], []

            for i in ids:
                c, t, r = i.split("_")
                conditions.append(c)
                times.append(t)
                replicates.append(r)

            return conditions, times, replicates

        if count == 1:

            conditions, times = [], []

            for i in ids:
                c, t = i.split("_")
                conditions.append(c)
                times.append(t)

            return conditions, times

        else:

            raise ValueError('Number of underscores is different from 2 or 3')


class StaticPlot(Plot):
    """
    Class to generate the different static plots.

    :param fmt: Output format of static plots (pdf, svg, png or jpeg)
    :type fmt: str
    :param display: Should plots be displayed when created
    :type display: Bool
    """

    def __init__(self, stack, value, data, name, metabolite,
                 condition, time, fmt, display=False):
        """

        """

        super().__init__(stack, value, data, name, metabolite, condition, time)
        self.fmt = fmt
        self.static_fig_name = self.metabolite + '.' + self.fmt
        self.display = display

    def stacked_areaplot(self):
        """Creation of area stackplot (for cinetic data)"""

        # Commençons par la préparation de data
        stackpivot = self.filtered_data.pivot(
            index='ID', columns='isotopologue', values=self.value)
        stackpivot = stackpivot.reindex(index=natsorted(stackpivot.index))
        pivotcol = stackpivot[0:].to_numpy()
        stackyval = pivotcol.transpose()
        stackxval = stackpivot.index.to_numpy()

        # Passons au plot
        plt.figure(figsize=[38, 20])
        labels = list(stackpivot.columns)
        plt.stackplot(stackxval,
                      stackyval,
                      labels=labels,
                      colors=cc.glasbey_dark[:len(
                          self.data['isotopologue'].unique())])
        plt.legend()
        plt.title("{} CID cinetics".format(self.metabolite))
        plt.xticks(rotation=45)
        plt.legend(loc='center left', bbox_to_anchor=(1, 0.5))
        plt.tight_layout()
        plt.savefig(self.static_fig_name, bbox_inches='tight', format=self.fmt)

        if self.display:
            plt.show()
        else:
            plt.close()

    def barplot(self):
        """Creation of barplots"""

        # Nous filtrons et préparons la table pour les donnéees
        mydata = self.filtered_data
        mydatapivot = mydata.pivot_table(index=["condition_order", 'ID'],
                                         columns='isotopologue',
                                         values=self.value)

        # Ici nous mettons les données dans l'ordre choisi par le template
        mydatapivot.sort_index(level="condition_order", inplace=True)
        mydatapivot = mydatapivot.droplevel(level="condition_order")

        # Passons au plot
        sns.set_context("poster")
        ax = mydatapivot.plot.bar(stacked=self.stack,
                                  figsize=(30, 15),
                                  title=self.metabolite,
                                  color=cc.glasbey_dark[:len(
                                      self.data['isotopologue'].unique())])
        ax.set_xlabel('Condition, Time and Replicate')
        ax.set_ylabel(self.value)
        ax.set_xticklabels(ax.get_xticklabels(),
                           rotation=45,
                           horizontalalignment='right')
        ax.legend(loc='center left', bbox_to_anchor=(1, 0.5))
        plt.tight_layout()
        plt.savefig(self.static_fig_name, bbox_inches='tight', format=self.fmt)
        if self.display:
            plt.show()
        else:
            plt.close()

    def mean_barplot(self):
        """Creation of meaned barplots (on replicates)"""

        # Ici nous faisons les moyennes et les SD des données et nous les mettons dans un df
        df_replicate_mean = self.filtered_data.groupby(
            ["condition_order", "condition", "time", "isotopologue"])[self.value].mean()
        df_replicate_std = self.filtered_data.groupby(
            ["condition_order", "condition", "time", "isotopologue"])[self.value].std()
        df_full = pd.concat([df_replicate_mean, df_replicate_std], axis=1)
        df_full.columns = ("mean", "std")

        # Nous formattons le df pour pouvoir plotter
        df_ready = df_full.unstack()
        df_ready.sort_index(level="condition_order", inplace=True)
        df_ready = df_ready.droplevel(level="condition_order")

        # Passons au plot
        sns.set_context("poster")
        colors = cc.glasbey_dark[:len(df_ready.columns)]
        this_ax = df_ready["mean"].plot.bar(stacked=self.stack,
                                            yerr=df_ready['std'],
                                            figsize=(30, 15),
                                            title=self.metabolite,
                                            color=colors)
        this_ax.set_xlabel('Condition, Time and Replicate')
        this_ax.set_ylabel(self.value)
        this_ax.set_xticklabels(this_ax.get_xticklabels(),
                                rotation=45,
                                horizontalalignment='right')
        this_ax.legend(loc='center left', bbox_to_anchor=(1, 0.5))
        plt.xticks(rotation=45)
        plt.tight_layout()
        plt.savefig(self.static_fig_name, bbox_inches='tight', format=self.fmt)
        if self.display:
            plt.show()
        else:
            plt.close()

    def mean_enrichment_plot(self):
        """Generate static mean_enrichment plots"""

        # Nous préparons une liste et un df dans lesquels on va ajouter les mean_enrichment
        list_of_tmpdfs = []

        # Nous retirons les duplicats pour chaque ID individuel
        for ID in self.filtered_data["ID"].drop_duplicates():
            tmpdf = self.filtered_data[self.filtered_data["ID"] == ID].drop_duplicates(
                subset=['mean_enrichment'])
            list_of_tmpdfs.append(tmpdf)

        # Nous mettons l'ordre de conditions en fonction du template, et nous préparons les datas pour plotter
        mean_enrichment_df = pd.concat(list_of_tmpdfs, ignore_index=True)
        mean_enrichment_df = mean_enrichment_df[["ID", "condition_order", "mean_enrichment"]]
        mean_enrichment_df.sort_values(by="condition_order", inplace=True)
        mean_enrichment_df.drop(labels="condition_order", axis=1, inplace=True)
        mean_enrichment_df.set_index("ID", inplace=True)

        # Nous plottons les data avec la fonction de pandas
        sns.set_context("poster")
        ax = mean_enrichment_df.plot.bar(figsize=(30, 15),
                                         title=self.metabolite,
                                         color=cc.glasbey_dark[3])
        ax.set_xlabel('Condition, Time and Replicate')
        ax.set_ylabel("mean_enrichment")
        ax.set_xticklabels(ax.get_xticklabels(), rotation=45, horizontalalignment='right')
        ax.legend(loc='center left', bbox_to_anchor=(1, 0.5))
        plt.tight_layout()
        plt.savefig(self.static_fig_name, bbox_inches='tight', format=self.fmt)
        if self.display:
            plt.show()
        else:
            plt.close()

    def mean_enrichment_meanplot(self):
        """Generate static mean_enrichment plots with meaned replicates"""

        # Nous préparons une liste et un df dans lesquels on va ajouter les mean_enrichment
        list_of_tmpdfs = []

        # Nous retirons les duplicats pour chaque ID individuel
        for ID in self.filtered_data["ID"].drop_duplicates():
            tmpdf = self.filtered_data[
                self.filtered_data["ID"] == ID].drop_duplicates(subset=['mean_enrichment'])
            list_of_tmpdfs.append(tmpdf)
        mean_enrichment_df = pd.concat(list_of_tmpdfs, ignore_index=True)

        # Ici nous faisons les moyennes et les SD des données et nous les mettons dans un df
        df_replicate_mean = mean_enrichment_df.groupby(
            ["condition_order", "condition", "time", "isotopologue"])['mean_enrichment'].mean()
        df_replicate_std = mean_enrichment_df.groupby(
            ["condition_order", "condition", "time", "isotopologue"])['mean_enrichment'].std()
        df_full = pd.concat([df_replicate_mean, df_replicate_std], axis=1)
        df_full.columns = ("mean", "std")

        # Nous formattons le df pour pouvoir plotter
        df_ready = df_full.unstack()
        df_ready.sort_index(level="condition_order", inplace=True)
        df_ready = df_ready.droplevel(level="condition_order")

        # Passons au plot
        sns.set_context("poster")
        this_ax = df_ready["mean"].plot.bar(yerr=df_ready['std'],
                                            figsize=(30, 15),
                                            title=self.metabolite,
                                            color=cc.glasbey_dark[3])
        this_ax.set_xlabel('Condition, Time and Replicate')
        this_ax.set_ylabel('mean_enrichment')
        this_ax.set_xticklabels(this_ax.get_xticklabels(),
                                rotation=45,
                                horizontalalignment='right')
        this_ax.legend(loc='center left', bbox_to_anchor=(1, 0.5))
        plt.xticks(rotation=45)
        plt.tight_layout()
        plt.savefig(self.static_fig_name, bbox_inches='tight', format=self.fmt)
        if self.display:
            plt.show()
        else:
            plt.close()


class InteractivePlot(Plot):
    """Class to generate the different interactive plots"""

    def __init__(self, stack, value, data, name, metabolite, condition, time):

        super().__init__(stack, value, data, name, metabolite, condition, time)
        self.filename = self.metabolite + ".html"
        self.plot_tools = "save, wheel_zoom, reset, hover, pan"

    def mean_enrichment_plot(self):
        """Generate interactive mean_enrichment plots"""

        output_file(filename=self.metabolite + ".html", title=self.metabolite)

        # Nous filtrons les données en fonction des paramètres du dashboard
        tmpdf = self.filtered_data

        # Nous préparons une liste et un df dans lesquels on va ajouter les mean_enrichment
        list_of_tmpdfs = []

        # Nous retirons les duplicats pour chaque ID individuel
        for ID in tmpdf["ID"].drop_duplicates():
            tmpdf2 = tmpdf[tmpdf["ID"] == ID].drop_duplicates(subset=['mean_enrichment'])
            list_of_tmpdfs.append(tmpdf2)

        # Nous mettons l'ordre de conditions en fonction du template, et nous préparons les datas pour plotter
        mean_enrichment_df = pd.concat(list_of_tmpdfs, ignore_index=True)

        del list_of_tmpdfs

        mean_enrichment_df = mean_enrichment_df[["ID", "condition_order", "mean_enrichment"]]
        mean_enrichment_df.sort_values(by="condition_order", inplace=True)
        mean_enrichment_df.drop(labels="condition_order", axis=1, inplace=True)
        mean_enrichment_df.set_index("ID", inplace=True)

        my_x_range = mean_enrichment_df.index.tolist()
        values = mean_enrichment_df["mean_enrichment"].tolist()
        conditions, times, replicates = Plot.split_ids(my_x_range)
        source = bk.models.ColumnDataSource(dict(x=my_x_range, y=values,
                                                 conds=conditions,
                                                 times=times,
                                                 reps=replicates))

        TOOLTIPS = [
            ("Condition", "@conds"),
            ("Time", "@times"),
            ("Replicate", "@reps"),
            ("Value", "@y")
        ]

        myplot = figure(plot_width=self.WIDTH,
                        plot_height=self.HEIGHT,
                        title=self.name,
                        y_axis_label="mean_enrichment",
                        tools=self.plot_tools,
                        tooltips=TOOLTIPS,
                        x_range=my_x_range)

        myplot.vbar(width=0.9,
                    bottom=0,
                    top="y",
                    color=cc.glasbey_dark[3],
                    x="x",
                    source=source)

        myplot.xaxis.major_label_orientation = math.pi / 4
        show(myplot)

    def mean_enrichment_meanplot(self):
        """Generate interactive mean_enrichment plots with meaned replicates"""

        output_file(filename=self.metabolite + ".html", title=self.metabolite)

        # Nous filtrons les données en fonction des paramètres du dashboard
        tmpdf = self.filtered_data

        df_replicate_mean = tmpdf.groupby(
            ["condition_order", "condition", "time", "isotopologue"])[self.value].mean()
        df_replicate_std = tmpdf.groupby(
            ["condition_order", "condition", "time", "isotopologue"])[self.value].std()
        mean_df = df_replicate_mean.to_frame()
        std_df = df_replicate_std.to_frame()

        # Ici nous mettons les moyennes dans un df et nous trions dans l'ordre demandé dans le template
        mean_df_unstack = mean_df.unstack()
        mean_df_unstack.sort_index(level="condition_order", inplace=True)
        mean_df_unstack = mean_df_unstack.droplevel(level=0)
        mean_df_unstack.columns = mean_df_unstack.columns.droplevel(level=0)
        mean_df_unstack.columns = mean_df_unstack.columns.astype(str)
        mean_df_unstack.index = ['{}_{}'.format(i, j) for i, j in mean_df_unstack.index]  # Nous recréons la colonne ID

        # Les colonnes contenants toutes les mêmes valeurs, nous pouvons juste prendre la première
        mean_series = mean_df_unstack.iloc[:, 0].copy()

        # Ici nous mettons les SD dans un df et nous trions dans l'ordre demandé dans le template
        std_df_unstack = std_df.unstack()
        std_df_unstack.sort_index(level="condition_order", inplace=True)
        std_df_unstack = std_df_unstack.droplevel(level=0)
        std_df_unstack.columns = std_df_unstack.columns.droplevel(level=0)
        std_df_unstack.columns = std_df_unstack.columns.astype(str)
        std_df_unstack.index = ['{}_{}'.format(i, j) for i, j in std_df_unstack.index]

        # Les colonnes contenants toutes les mêmes valeurs, nous pouvons juste prendre la première
        std_series = std_df_unstack.iloc[:, 0].copy()

        # Nous préparons les hauts et bas pour placer les barres d'erreur
        upper_series = mean_series.add(std_series, fill_value=0)
        upper_list = upper_series.to_list()

        lower_series = mean_series.sub(std_series, axis='index', fill_value=0)
        lower_list = lower_series.to_list()

        # Nous préparons les datas pour plotter
        my_x_range = mean_series.index.tolist()
        values = mean_series.to_list()
        conditions, times, replicates = Plot.split_ids(my_x_range)
        my_dict = dict(ID=my_x_range, tops=values,
                       conds=conditions, times=times,
                       reps=replicates)
        source = bk.models.ColumnDataSource(my_dict)

        # Nous préparons le dictionnaire qui va faire le ColumnDataSource pour les barres d'erreur
        whisker_dico = dict(base=my_x_range, upper=upper_list, lower=lower_list)

        # Passons au plot
        TOOLTIPS = [
            ("Condition", "@conds"),
            ("Time", "@times"),
            ("Replicate", "@reps"),
            ("Value", "@tops")
        ]

        myplot = figure(plot_width=self.WIDTH,
                        plot_height=self.HEIGHT,
                        title=self.name,
                        y_axis_label="mean_enrichment",
                        tools=self.plot_tools,
                        tooltips=TOOLTIPS,
                        x_range=my_x_range)

        myplot.vbar(width=0.9,
                    bottom=0,
                    top='tops',
                    color=cc.glasbey_dark[3],
                    x='ID',
                    source=source)

        mywhisker = bk.models.Whisker(source=bk.models.ColumnDataSource(whisker_dico),
                                      base="base",
                                      upper="upper",
                                      lower="lower",
                                      level="overlay")

        # Nous ajoutons les barres d'erreurs
        myplot.add_layout(mywhisker)

        myplot.xaxis.major_label_orientation = math.pi / 4
        show(myplot)

    def stacked_barplot(self):
        """Generate interactive stacked barplots"""

        output_file(filename=self.metabolite + ".html", title=self.metabolite)

        # Nous filtrons les datas à plotter
        mydata = self.filtered_data
        mydatapivot = mydata.pivot_table(index=["condition_order", 'ID'],
                                         columns='isotopologue',
                                         values=self.value)
        mydatapivot.sort_index(level="condition_order", inplace=True)
        mydatapivot = mydatapivot.droplevel(level="condition_order")
        mydatapivot.columns = mydatapivot.columns.astype(str)

        # préparons les différentes couches des barres à stacker
        stackers = mydatapivot.columns.tolist()

        # Récupérons les noms à mettre en x
        my_x_range = mydatapivot.index.tolist()

        # Nous récupérons les noms pour les tooltips
        conditions, times, replicates = Plot.split_ids(my_x_range)

        # Nous faisons ici des listes avec les données pour chaque couche (une couche=un isotopologue)
        listoflists = [mydatapivot[val].tolist() for val in stackers]

        # Nous mettons ça dans un dictionnaire pour le ColumnDataSource
        myplotdic = dict(zip(stackers, listoflists))
        myplotdic.update({'ID': my_x_range,
                          'conds': conditions,
                          'times': times,
                          'reps': replicates})

        # Préparation des tooltips
        TOOLTIPS = [
            ("Condition", "@conds"),
            ('Time', '@times'),
            ('Replicate', '@reps'),
            ("Isotopologue", "$name"),
            ("Value", "@$name"),
        ]

        # Initialization de la figure
        myplot = figure(
            x_range=my_x_range,
            plot_width=self.WIDTH,
            plot_height=self.HEIGHT,
            title=self.name,
            y_axis_label=self.value,
            tools=self.plot_tools,
            tooltips=TOOLTIPS
        )

        # Passons au plot
        myplot.vbar_stack(stackers,
                          x='ID',
                          source=myplotdic,
                          width=0.9,
                          color=cc.glasbey_dark[:len(stackers)]
                          )

        myplot.xaxis.major_label_orientation = math.pi / 4

        show(myplot)

    def unstacked_barplot(self):
        """Generate interactive unstacked barplots"""

        output_file(filename=self.metabolite + ".html", title=self.metabolite)

        # Nous filtrons les datas à plotter
        mydata = self.filtered_data
        mydatapivot = mydata.pivot_table(index=["condition_order", 'ID'],
                                         columns='isotopologue',
                                         values=self.value)
        mydatapivot.sort_index(level="condition_order", inplace=True)
        mydatapivot = mydatapivot.droplevel(level="condition_order")

        # Nous récupérons les colonnes pour faire les couches à stacker
        stackers = mydatapivot.columns.astype(str).tolist()

        # Nous faisons des tuples avec index et couche à plotter
        factors = [(i, stack) for i in mydatapivot.index for stack in stackers]
        condition_time = [i[0] for i in factors]
        isotops = [i[1] for i in factors]

        # Nous récupérons les noms pour les tooltips
        conditions, times, replicates = Plot.split_ids(condition_time)

        # Nous récupérons les valeurs de chaque couche
        tops = [row[int(stack) + 1]
                for row in mydatapivot.itertuples()
                for stack in stackers]

        # Nous mettons tout ça dans un ColumnDataSource
        source = bk.models.ColumnDataSource(data=dict(
            x=factors, tops=tops, conds=conditions,
            times=times, reps=replicates, isotops=isotops))

        # Préparation des tooltips
        TOOLTIPS = [
            ("Condition", "@conds"),
            ("Time", '@times'),
            ("Replicate", "@reps"),
            ('Isotopologue', '@isotops'),
            ("Value", "@tops")
        ]

        # Initialisation de la figure
        plot = figure(
            x_range=bk.models.FactorRange(*factors),  # Voir docu sur Bokeh.org pour ça
            plot_width=self.WIDTH,
            plot_height=self.HEIGHT,
            tools=self.plot_tools,
            tooltips=TOOLTIPS)

        # Passons au plot
        plot.vbar(x='x',
                  top='tops',
                  width=0.9,
                  source=source,
                  fill_color=bk.transform.factor_cmap('x',
                                                      palette=cc.glasbey_dark,
                                                      factors=stackers,
                                                      start=1, end=2),
                  line_color="white")

        plot.xaxis.major_label_orientation = math.pi / 4
        plot.y_range.start = 0
        plot.x_range.range_padding = 0.1
        show(plot)

    def stacked_meanplot(self):
        """Generate interactive stacked barplots with meaned replicates"""

        output_file(filename=self.metabolite + ".html", title=self.metabolite)

        # Nous filtrons les datas à plotter et préparons les moyennes et SD
        tmpdf = self.filtered_data
        df_replicate_mean = tmpdf.groupby(
            ["condition_order", "condition", "time", "isotopologue"])[self.value].mean()
        df_replicate_std = tmpdf.groupby(
            ["condition_order", "condition", "time", "isotopologue"])[self.value].std()
        mean_df = df_replicate_mean.to_frame()
        std_df = df_replicate_std.to_frame()

        # Ici nous mettons les moyennes et SD dans des df
        mean_df_unstack = mean_df.unstack()
        mean_df_unstack.sort_index(level="condition_order", inplace=True)
        mean_df_unstack = mean_df_unstack.droplevel(level=0)
        mean_df_unstack.columns = mean_df_unstack.columns.droplevel(level=0)
        mean_df_unstack.columns = mean_df_unstack.columns.astype(str)
        mean_df_unstack.index = ['{}_{}'.format(i, j) for i, j in mean_df_unstack.index]

        std_df_unstack = std_df.unstack()
        std_df_unstack.sort_index(level="condition_order", inplace=True)
        std_df_unstack = std_df_unstack.droplevel(level=0)
        std_df_unstack.columns = std_df_unstack.columns.droplevel(level=0)
        std_df_unstack.columns = std_df_unstack.columns.astype(str)
        std_df_unstack.index = ['{}_{}'.format(i, j) for i, j in std_df_unstack.index]

        upper_df = mean_df_unstack.add(std_df_unstack, fill_value=0)
        upper_df.columns = upper_df.columns.astype(str)
        lower_df = mean_df_unstack.sub(std_df_unstack, axis='index', fill_value=0)
        lower_df.columns = lower_df.columns.astype(str)

        stackers = mean_df_unstack.columns.tolist()
        my_x_range = mean_df_unstack.index.tolist()

        # Nous initialisons les listes de valeurs moyennes isotopologue par isotopologue
        mean_listoflists = [mean_df_unstack[val].tolist() for val in stackers]

        meanplotdic = dict(zip(stackers, mean_listoflists))
        meanplotdic.update({'ID': my_x_range})

        # Nous récupérons les noms pour les tooltips
        conditions, times = Plot.split_ids(my_x_range)

        meanplotdic.update({'conds': conditions,
                            'times': times})

        # Préparation des tooltips
        TOOLTIPS = [
            ("Condition", "@conds"),
            ("Time", "@times"),
            ("Isotopologue", "$name"),
            ("Value", "@$name"),
        ]

        # Initialization de la figure
        myplot = figure(
            x_range=my_x_range,
            plot_width=self.WIDTH,
            plot_height=self.HEIGHT,
            title=self.name,
            y_axis_label=self.value,
            tools=self.plot_tools,
            tooltips=TOOLTIPS
        )

        colors = cc.glasbey_dark[:len(stackers)]

        # Passons au plot
        myplot.vbar_stack(stackers,
                          x='ID',
                          source=meanplotdic,
                          width=0.9,
                          color=colors
                          )

        # Nous préparons les données pour positionner les barres d'erreur
        dico_upper_list = []
        dico_lower_list = []

        for _ in upper_df[stackers[0]]:
            dico_upper_list.append(0)
            dico_lower_list.append(0)

        dico_upper_array = np.array(dico_upper_list)
        dico_lower_array = np.array(dico_lower_list)

        # Nous faisons la suite couche par couche
        for val in stackers:
            base = my_x_range

            # Nous devons incrementer à chaque loop la valeur pour la barre d'erreur du dessous pour avoir la bonne
            # hauteur de placement
            upper_listoflists = list(upper_df[val])
            lower_listoflists = list(lower_df[val])
            upper_array = np.array(upper_listoflists)
            lower_array = np.array(lower_listoflists)
            dico_upper_array = np.add(dico_upper_array, upper_array)
            dico_lower_array = np.add(dico_lower_array, lower_array)
            dico_lower_list = dico_lower_array.tolist()
            dico_upper_list = dico_upper_array.tolist()
            whisker_dico = dict(base=base, upper=dico_upper_list, lower=dico_lower_list)
            source = bk.models.ColumnDataSource(data=whisker_dico)
            mywhisker = Whisker(source=source, base="base", upper="upper", lower="lower", level="overlay")

            # Nous ajoutons les barres d'erreurs (couche par couche)
            myplot.add_layout(mywhisker)

        myplot.xaxis.major_label_orientation = math.pi / 4
        show(myplot)

    def unstacked_meanplot(self):
        """Generate interactive unstacked barplots with meaned replicates"""

        output_file(filename=self.metabolite + ".html", title=self.metabolite)

        # Préparation des datas à plotter
        tmpdf = self.filtered_data
        df_replicate_mean = tmpdf.groupby(
            ["condition_order", "condition", "time", "isotopologue"])[self.value].mean()
        df_replicate_std = tmpdf.groupby(
            ["condition_order", "condition", "time", "isotopologue"])[self.value].std()
        mean_df = df_replicate_mean.to_frame()
        std_df = df_replicate_std.to_frame()

        mean_df_unstack = mean_df.unstack()
        mean_df_unstack.sort_index(level="condition_order", inplace=True)
        mean_df_unstack = mean_df_unstack.droplevel(level=0)
        mean_df_unstack.columns = mean_df_unstack.columns.droplevel(level=0)
        mean_df_unstack.columns = mean_df_unstack.columns.astype(str)
        mean_df_unstack.index = ['{}_{}'.format(i, j) for i, j in mean_df_unstack.index]

        std_df_unstack = std_df.unstack()
        std_df_unstack.sort_index(level="condition_order", inplace=True)
        std_df_unstack = std_df_unstack.droplevel(level=0)
        std_df_unstack.columns = std_df_unstack.columns.droplevel(level=0)
        std_df_unstack.columns = std_df_unstack.columns.astype(str)
        std_df_unstack.index = ['{}_{}'.format(i, j) for i, j in std_df_unstack.index]
        stackers = mean_df_unstack.columns.tolist()

        upper_df = mean_df_unstack.add(std_df_unstack, fill_value=0)
        lower_df = mean_df_unstack.sub(std_df_unstack, axis='index', fill_value=0)

        # Même principe que pour les non stackés non moyennés sauf que nous préparons les barres d'erreur aussi
        factors = [(i, stack) for i in mean_df_unstack.index for stack in stackers]
        condition_time = [i[0] for i in factors]
        isotops = [i[1] for i in factors]
        conditions, times = Plot.split_ids(condition_time)

        tops = [row[int(stack) + 1]
                for row in mean_df_unstack.itertuples()
                for stack in stackers]

        upper_list = [row[int(stack) + 1]
                      for row in upper_df.itertuples()
                      for stack in stackers]

        lower_list = [row[int(stack) + 1]
                      for row in lower_df.itertuples()
                      for stack in stackers]

        base = factors
        source = bk.models.ColumnDataSource(data=dict(
            x=factors, tops=tops, conds=conditions,
            times=times, isotops=isotops))

        # Nous faisons un ColumnDataSource pour les incertitudes
        source_error = bk.models.ColumnDataSource(
            data=dict(base=base, upper=upper_list, lower=lower_list))

        # Préparation des tooltips
        TOOLTIPS = [
            ("Condition", "@conds"),
            ("Time", "@times"),
            ("Isotopologue", "@isotops"),
            ("Fraction", "@tops")
        ]

        # Initialisation de la figure
        plot = figure(
            x_range=bk.models.FactorRange(*factors),
            plot_width=self.WIDTH,
            plot_height=self.HEIGHT,
            tools=self.plot_tools,
            tooltips=TOOLTIPS)

        # Passons au plot
        plot.vbar(x='x',
                  top='tops',
                  width=0.9,
                  source=source,
                  fill_color=bk.transform.factor_cmap(
                      'x', palette=cc.glasbey_dark,
                      factors=stackers, start=1, end=2),
                  line_color="white")

        plot.add_layout(bk.models.Whisker(
            source=source_error,
            base="base", upper="upper",
            lower="lower", level="overlay"))
        plot.y_range.start = 0
        plot.x_range.range_padding = 0.1
        show(plot)

    def stacked_areaplot(self):
        """Generate interactive stacked areaplots"""

        output_file(filename=self.metabolite + ".html", title=self.metabolite)

        # Commençons par la préparation de data
        stackdf = self.filtered_data
        stackpivot = stackdf.pivot(index='ID', columns='isotopologue', values=self.value)
        stackpivot.columns = stackpivot.columns.astype(str)
        mysource = bk.models.ColumnDataSource(data=stackpivot)
        mystackers = stackpivot.columns.tolist()
        colors = cc.glasbey_dark[:len(mystackers)]

        TOOLTIPS = [
            ("", "<strong>@ID</strong>"),
            ("isotopologue", "$name"),
            ("ID", "@$name"),
        ]

        plot = figure(
            width=self.WIDTH,
            height=self.HEIGHT,
            tools=self.plot_tools,
            tooltips=TOOLTIPS,
            x_range=stackpivot.index.values
        )

        plot.xaxis.major_label_orientation = math.pi / 4
        plot.varea_stack(mystackers, x="ID", color=colors, source=mysource)
        show(plot)


class Map:
    """
    Class to create maps from Isocor output (MS data from C13 labelling experiments)

    :param annot: Should annotations be apparent on map or not
    :type annot: Bool
    """

    def __init__(self, data, name, annot, fmt, display=False):
        """

        """
        self.data = data
        self.name = name
        self.annot = annot
        self.fmt = fmt
        self.display = display

        # Il faut préparer les données pour les maps:
        self.heatmapdf = self.data[
            ['metabolite', 'mean_enrichment',
             'condition', 'time', 'number_rep']]
        self.heatmapdf = self.heatmapdf[self.heatmapdf['mean_enrichment'] != 0]
        self.heatmapdf.drop_duplicates(inplace=True)
        self.heatmapdf.dropna(inplace=True)
        self.heatmapdf.set_index('metabolite', inplace=True)
        self.heatmapdf['Condition_Time'] = self.heatmapdf['condition'].apply(str) + '_T' + self.heatmapdf['time'].apply(
            str)
        self.heatmapdf = self.heatmapdf.groupby(
            ['metabolite', 'Condition_Time'])['mean_enrichment'].mean()
        self.heatmapdf = self.heatmapdf.unstack(0)
        self.dc_heatmap = self.heatmapdf.describe(include='all')
        self.heatmap_center = self.dc_heatmap.iloc[5, 0].mean()
        self.clustermapdf = self.heatmapdf.fillna(value=0)

    def build_heatmap(self):
        """
        Create a heatmap of mean_enrichment data across
        all conditions & times & metabolites
        """

        fig, ax = plt.subplots(figsize=(30, 30))
        sns.set(font_scale=1)
        sns.heatmap(self.heatmapdf, vmin=0.02,
                    robust=True, center=self.heatmap_center,
                    annot=self.annot, fmt="f", linecolor='black',
                    linewidths=.2, cmap='Blues', ax=ax)
        plt.yticks(rotation=0, fontsize=20)
        plt.xticks(rotation=45, fontsize=20)
        # bottom, top = ax.get_ylim()
        plt.savefig(self.name + '.' + self.fmt, bbox_inches='tight', format=self.fmt)
        if self.display:
            plt.show()

    def build_clustermap(self):
        """
        Create a clustermap of mean_enrichment data across
        all conditions & times & metabolites
        """

        sns.set(font_scale=1)
        cg = sns.clustermap(self.clustermapdf,
                            cmap="Blues", fmt="f",
                            linewidths=.2, standard_scale=1,
                            figsize=(30, 30), linecolor='black',
                            annot=self.annot)
        plt.setp(cg.ax_heatmap.yaxis.get_majorticklabels(), rotation=0, fontsize=20)
        plt.setp(cg.ax_heatmap.xaxis.get_majorticklabels(), rotation=45, fontsize=20)
        plt.savefig(self.name + '.' + self.fmt, bbox_inches='tight', format=self.fmt)
        if self.display:
            plt.show()

    def build_interactive_heatmap(self):
        """
        Create an interactive heatmap of mean_enrichment data across
        all conditions & times & metabolites
        """

        output_file(filename=self.name + ".html", title=self.name + ".html")

        condition_time = list(self.heatmapdf.index.astype(str))
        metabolites = list(self.heatmapdf.columns)

        # Nous réordonnons les données
        df = pd.DataFrame(self.heatmapdf.stack(), columns=["values"]).reset_index()

        # Nous préparons les couleurs
        colors = cc.kbc[len(df)]
        mapper = LinearColorMapper(palette=cc.kbc[:len(df)])

        TOOLTIPS = "hover,save"

        # initialisation de la figure
        p = figure(title=self.name,
                   x_range=list(reversed(metabolites)), y_range=condition_time,
                   x_axis_location="below", plot_width=1080, plot_height=640,
                   tools=TOOLTIPS, toolbar_location='above',
                   tooltips=[('datapoint', '@metabolite @Condition_Time'), ('value', "@values")])

        p.grid.grid_line_color = None
        p.axis.axis_line_color = None
        p.axis.major_tick_line_color = None
        p.axis.major_label_text_font_size = "15px"
        p.axis.major_label_standoff = 0
        p.xaxis.major_label_orientation = math.pi / 3

        # Passons au plot
        p.rect(x="metabolite",
               y="Condition_Time",
               width=1, height=1,
               source=df,
               fill_color={'field': "values", 'transform': mapper},
               line_color=None)

        # Nous préparons la barre de couleur pour la légende
        color_bar = ColorBar(color_mapper=mapper, major_label_text_font_size="7px",
                             ticker=BasicTicker(desired_num_ticks=len(colors)),
                             formatter=PrintfTickFormatter(),
                             label_standoff=6, border_line_color=None, location=(0, 0))

        # Ajoutons la barre de couleur de la légende
        p.add_layout(color_bar, 'right')

        show(p)



import logging

import pandas as pd
from natsort import natsorted


class IsoplotData:
    """
    Class to prepare Isoplot Data for plotting

    
    :param datapath: Path to .csv file containing Isocor output data
    :type datapath: str
    :param isoplot_logger: Logger object
    :type isoplot_logger: child of logger class: 'logging.Logger'
    :param data: Isocor output data before merge
    :type data: Pandas DataFRame
    :param template: User-defined template containing conditions, times,
                    replicate numbers, etc... before merge
    :type template: Pandas DataFrame
    :param dfmerge: Dataframe containing merged data (template and isocor output). 
                    Is called by prepare_data method and used by Plot class
    :type dfmerge: Pandas DataFrame
    
    :raises UnicodeDecodeError: Is raised by get_template method when template file is 
                                not .xlsx or encoding is not utf-8
    :raises AssertionError: return error if after merge data is not of tye DataFrame
    
    """

    def __init__(self, datapath):

        self.datapath = datapath
        self.data = None
        self.template = None
        self.dfmerge = None

        self.isoplot_logger = logging.getLogger(__name__)
        stream_handle = logging.StreamHandler()
        formatter = logging.Formatter(
            '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        stream_handle.setFormatter(formatter)
        self.isoplot_logger.addHandler(stream_handle)

        self.isoplot_logger.debug('Initializing IsoplotData object')
        self.isoplot_logger.info('Reading datafile {} \n'.format(
            self.datapath))

    def get_data(self):

        """Read data from csv file and store in object data attribute."""

        self.isoplot_logger.info("Reading file...")

        try:
            self.isoplot_logger.debug('Trying to open csv with ";" separator')
            self.data = pd.read_csv(self.datapath, sep=';')
        except Exception as _:
            try:
                self.isoplot_logger.debug('Trying to open csv with "\t" separator')
                self.data = pd.read_csv(self.datapath, sep='\t')
            except Exception as err:
                self.isoplot_logger.error(
                    'There was a problem reading the csv file. Check that format is .csv with ";" or tabulated '
                    'separator')
                self.isoplot_logger.error(err)
        else:
            if type(self.data) is None:
                self.isoplot_logger.error(
                    'Dataframe is empty. Please check that file is .csv with ";" or tabulated separator')

    def generate_template(self):
        """Generate .xlsx template that user must fill"""

        self.isoplot_logger.info("Generating template...")

        metadata = pd.DataFrame(columns=[
            "sample", "condition", "condition_order", "time", "number_rep", "normalization"])
        metadata["sample"] = natsorted(self.data["sample"].unique())
        metadata["condition"] = 'votre_condition'
        metadata["condition_order"] = 1
        metadata["time"] = 1
        metadata["number_rep"] = 3
        metadata["normalization"] = 1.0
        metadata.to_excel(r'ModifyThis.xlsx', index=False)

        self.isoplot_logger.info('Template has been generated')

    def get_template(self, path):
        """Read user-filled template and catch any encoding errors"""

        self.isoplot_logger.info("Reading template...")

        try:
            self.isoplot_logger.debug('Trying to read excel template')
            self.template = pd.read_excel(path, engine='openpyxl')

        except UnicodeDecodeError as uni:
            self.isoplot_logger.error(uni)
            self.isoplot_logger.error(
                'Unable to read file. Check file encoding (must be utf-8) or file format (format must be .xlsx)')

        except Exception as err:
            self.isoplot_logger.error("There has been a problem...")
            self.isoplot_logger.error(err)

        else:
            self.isoplot_logger.info("Template succesfully loaded")

    def merge_data(self):
        """Merge template and data into pandas dataframe """

        self.isoplot_logger.info("Merging into dataframe...")

        try:
            self.isoplot_logger.debug('Trying to merge datas')
            self.dfmerge = self.data.merge(self.template)

            if not isinstance(self.dfmerge, pd.DataFrame):
                raise TypeError(
                    f"Error while merging data, dataframe not created. Data turned out to be {type(self.dfmerge)}")

        except Exception as err:
            self.isoplot_logger.error(err)
            self.isoplot_logger.error(
                'Merge impossible. Check column headers or file format (format must be .xlsx)')
            raise
            
        else:
            self.isoplot_logger.info('Dataframes have been merged')

    def prepare_data(self):
        """Final cleaning of data and export"""

        self.isoplot_logger.debug('Preparing data after merge: normalizing...')

        self.dfmerge["corrected area normalized"] = self.dfmerge["corrected_area"] / self.dfmerge["normalization"]
        self.dfmerge['metabolite'].drop_duplicates()

        self.isoplot_logger.debug("Creating IDs...")

        # Nous créons ici une colonne pour identifier chaque ligne avec condition+temps+numero de répétition (possibilité de rajouter un tag metabolite plus tard si besoin)
        self.dfmerge['ID'] = self.dfmerge['condition'].apply(str) + '_T' + self.dfmerge['time'].apply(str) + '_' + \
                             self.dfmerge['number_rep'].apply(str)

        self.isoplot_logger.debug('Applying final transformations...')
        # Vaut mieux ensuite retransformer les colonnes temps et number_rep en entiers pour éviter des problèmes éventuels de type
        self.dfmerge['time'].apply(int)
        self.dfmerge['number_rep'].apply(int)
        self.dfmerge.sort_values(['condition_order', 'condition'], inplace=True)
        self.dfmerge.to_excel(r'Data Export.xlsx', index=False)
        self.isoplot_logger.info('Data exported. Check Data Export.xlsx')



"""Isoplot2 cli module containing argument parser, cli initialization and main logger creation."""

import datetime
import os
import logging

from isoplot.utilities import control_isoplot_plot, control_isoplot_map, get_cli_input, parseArgs
from isoplot.dataprep import IsoplotData


def initialize_cli():
    """Initialize parser and call main function"""

    # Initiate logger
    logger = logging.getLogger(__name__)
    stream_handle = logging.StreamHandler()
    formatter = logging.Formatter(
        '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    stream_handle.setFormatter(formatter)
    logger.addHandler(stream_handle)

    parser = parseArgs()
    args = parser.parse_args()

    # Check for typos and input errors
    if args.format not in ['png', 'svg', 'pdf', 'jpeg', 'html']:
        raise RuntimeError("Format must be png, svg, pdf, jpeg or html")

    if not os.path.exists(args.destination):
        raise RuntimeError(
            f"The entered destination path {args.destination} is not valid. Please check for typos")

    # Set verbosity
    if args.verbose:
        logger.setLevel(logging.DEBUG)

    else:
        logger.setLevel(logging.INFO)

    main(args, logger)


def main(args, logger):
    """
    Main function responsible for directory creation, launching plot creation
    and coordinating different modules. Can be put in a separate module if the need
    ever arises (if a GUI is created for example).

    :param args: List of strings to parse. The default is taken from sys.argv
    :type args: list of arguments
    :param logger: main logger that will be derived in other modules
    :type logger: class: 'logging.Logger'

    :raises RuntimeError: if inputed destination path does not exist raises error
    :raises RuntimeError: if inputed template path does not exist raises error
     """

    logger.debug("Starting main")

    # Prepare directory where logger, plots and data will be exported
    now = datetime.datetime.now()
    date_time = now.strftime("%d%m%Y_%H%M%S")  # Récupération date et heure

    logger.debug("Building end folder")

    os.chdir(args.destination)  # Go to destination
    os.mkdir(args.name + " " + date_time)  # Create dir
    os.chdir(args.name + " " + date_time)  # Infiltrate dir

    logger.debug("Generating data object")

    # Initialize data object from path retrieved from user through argument parser
    try:
        data_object = IsoplotData(args.datafile)
        data_object.get_data()

    except Exception as dataload_err:
        raise RuntimeError(f"Error while loading data. \n Error: {dataload_err}")

    # If template is not given, we generate it and stop here
    if args.template == 0:
      
        logger.debug("Generating template")
        data_object.generate_template()
        logger.info("Template has been generated. Check destination folder at {}".format(args.destination))

    # If template is given, we check the path and then generate data object
    else:

        logger.debug("Getting template, merging and preparing data")

        if not os.path.exists(args.template):
            raise RuntimeError(f"Error in template path {args.template}")

        # Fetch template and merge with data
        try:
            data_object.get_template(args.template)

        except Exception as temp_err:
            raise RuntimeError(f"Error while getting template file.\n Error: {temp_err}")

        try:
            data_object.merge_data()

        except Exception as merge_err:
            raise RuntimeError(f"Error while merging data. \n Error: {merge_err}")

        try:
            data_object.prepare_data()

        except Exception as prep_err:
            raise RuntimeError(f"Error during final preparation of data.\n Error: {prep_err}")

        logger.debug("Getting input from cli")

        # Get lists of parameters for plots
        metabolites = get_cli_input(args.metabolite, "metabolite", data_object, logger)

        conditions = get_cli_input(args.condition, "condition", data_object, logger)

        times = get_cli_input(args.time, "time", data_object, logger)

        logger.info("--------------------")
        logger.info("metabolites: {}".format(metabolites))
        logger.info("conditions: {}".format(conditions))
        logger.info("times: {}".format(times))
        logger.info("--------------------")
        
        logger.info("Creating plots...")

        # Finally we call the function that coordinates the plot creation
        if args.static_heatmap or args.static_clustermap or args.interactive_heatmap:

            try:
                control_isoplot_map(args, data_object)

            except Exception as map_err:
                raise RuntimeError(f"Error while generating map.\n Error: {map_err}")

        else:
            for metabolite in metabolites:
                control_isoplot_plot(args, data_object, metabolite, conditions, times)

        logger.info('Done!')



import io
import datetime
import os

import ipywidgets as widgets

from isoplot.dataprep import IsoplotData
from isoplot.plots import StaticPlot, InteractivePlot, Map


class ValueHolder():
    x: int = None
vh = ValueHolder()


#Instanciation des widgets
def make_uploader():
    global uploader
    uploader = widgets.FileUpload(
        accept='',  # Accepted file extension e.g. '.txt', '.pdf', 'image/*', 'image/*,.pdf'
        multiple=False  # True to accept multiple files upload else False
    )
    
    return uploader
    
def make_mduploader():
    global mduploader
    mduploader = widgets.FileUpload(
        accept='',  # Accepted file extension e.g. '.txt', '.pdf', 'image/*', 'image/*,.pdf'
        multiple=False  # True to accept multiple files upload else False
    )

    return mduploader

metadatabtn = widgets.Button(description='Create Template')

datamerge_btn = widgets.Button(description='Submit Template')

out = widgets.Output()

out2 = widgets.Output()
    
def metadatabtn_eventhandler(event):
    global data_object
    
    with out:
        print("Loading file...")
    
    uploaded_filename = next(iter(uploader.value))
    content = uploader.value[uploaded_filename]['content']
    with open('myfile', 'wb') as f: f.write(content)
        
    data_object = IsoplotData(io.BytesIO(content))
    data_object.get_data()
    data_object.generate_template()
    
    with out:
        print("Done!")
        
def dataprep_eventhandler(event):
    
    with out2:
        print('Loading file...')
    
    mduploaded_filename = next(iter(mduploader.value))
    content = mduploader.value[mduploaded_filename]['content']
    with open('myfile', 'wb') as f: f.write(content)
    
    data_object.get_template(io.BytesIO(content))
    data_object.merge_data()
    data_object.prepare_data()
    vh.dfmerge = data_object.dfmerge
    
    with out2:
        print("Done!")
    
#Gestion des événements pour les boutons    
metadatabtn.on_click(metadatabtn_eventhandler)
datamerge_btn.on_click(dataprep_eventhandler)

#Fonction permettant le filtrage des données à plotter et appelant les fonctions de plotting
def indiplot (stack, value, data, name, metabolites, conditions, times, fmt, stackplot=False):
    
    #Préparons le directory où seront enregistrés les html avec les plots
    now = datetime.datetime.now()
    date_time = now.strftime("%d%m%Y_%H%M%S") #Récupération date et heure
    mydir = os.getcwd()
    os.mkdir(name + " " + date_time) #Créons le dir
    os.chdir(name + " " + date_time) #Rentrons dans le dir
    
    for metabolite in metabolites:
        
        plotter = StaticPlot(stack, value, data, name, metabolite, conditions, times, fmt)
        plotter.display = True
        
        if value != 'mean_enrichment':
            if stackplot == True:
                plotter.stacked_areaplot()
            else:
                plotter.barplot()
        elif value == 'mean_enrichment':
            plotter.plot_static_mean_enrichment_plot()

    os.chdir(mydir) #Revenir au dir initial
    
#Fonction permettant le filtrage des données à plotter et appelant les fonctions de plotting
def meanplot(stack, value, data, name, metabolites, conditions, times, fmt):
    
    now = datetime.datetime.now()
    date_time = now.strftime("%d%m%Y_%H%M%S") #Récupération date et heure
    mydir = os.getcwd()
    os.mkdir(name + " " + date_time) #Créons le dir
    os.chdir(name + " " + date_time) #Rentrons dans le dir
   
    for metabolite in metabolites:
        
        plotter = StaticPlot(stack, value, data, name, metabolite, conditions, times, fmt)
        plotter.display = True
        
        if value != 'mean_enrichment':
            plotter.mean_barplot()
        elif value == 'mean_enrichment':
            plotter.static_mean_enrichment_meanplot()

    os.chdir(mydir) #Revenir au dir initial  
    
#Création d'une fonction pour gérer les appels aux fonctions de plotting en individuel
def indibokplot(stack, value, data, name, metabolites, conditions, times, stackplot=False):
    
    #Préparons le directory où seront enregistrés les html avec les plots
    now = datetime.datetime.now()
    date_time = now.strftime("%d%m%Y_%H%M%S") #Récupération date et heure
    mydir = os.getcwd()
    os.mkdir(name + " " + date_time) #Créons le dir
    os.chdir(name + " " + date_time) #Rentrons dans le dir

    for metabolite in metabolites:
        
        if name == '':
            name = metabolite
        
        plotter = InteractivePlot(stack, value, data, name, metabolite, conditions, times)
        plotter.display = True
        
        if value != 'mean_enrichment': #Le cas du mean enrichment est différent car les valeurs sont en double à la sortie d'Isocor   
            
            if stackplot == True:
                plotter.stacked_areaplot()
            
            elif stack == False:
                plotter.unstacked_barplot()
                
            elif stack == True:
                plotter.stacked_barplot()
        
        elif value == 'mean_enrichment':
            plotter.mean_enrichment_plot()
            
    os.chdir(mydir) #Revenir au dir initial
    
#Création d'une fonction pour gérer les appels aux fonctions de plotting en individuel
def meanbokplot(stack, value, data, name, metabolites, conditions, times):
    
    #Préparons le directory où seront enregistrés les html avec les plots
    now = datetime.datetime.now()
    date_time = now.strftime("%d%m%Y_%H%M%S") #Récupération date et heure
    mydir = os.getcwd()
    os.mkdir(name + " " + date_time) #Créons le dir
    os.chdir(name + " " + date_time) #Rentrons dans le dir

    for metabolite in metabolites:
        
        if name == '':
            name = metabolite
        
        plotter = InteractivePlot(stack, value, data, name, metabolite, conditions, times)
        plotter.display = True
        
        if value != 'mean_enrichment': #Le cas du mean enrichment est différent car les valeurs sont en double à la sortie d'Isocor   
            
            if stack == False:
                plotter.unstacked_meanplot()
                
            elif stack == True:
                plotter.stacked_meanplot()
        
        elif value == 'mean_enrichment':
            plotter.mean_enrichment_meanplot()
            
    os.chdir(mydir) #Revenir au dir initial
    
#Fontion pour choisir le map à générer:
def build_map(data, name, map_select, annot, fmt):
    
    mapper = Map(data, name, annot, fmt)
    mapper.display == True
    
    if map_select == "Static heatmap":
        mapper.build_heatmap()
        
    if map_select == 'Interactive heatmap':
        mapper.fmt = 'html'
        mapper.build_interactive_heatmap()
        
    if map_select == "Clustermap":
        mapper.build_clustermap()


import argparse
import logging

from isoplot.plots import StaticPlot, InteractivePlot, Map

logger = logging.getLogger(__name__)


def parseArgs():
    """
    :return: Argument Parser object
    :rtype: class: argparse.ArgumentParser
    """

    logger.debug('Initializing parser')
    parser = argparse.ArgumentParser("Isoplot2: Plotting isotopic labelling MS data")

    logger.debug('Adding positionnal arguments for general plot infos')
    parser.add_argument('datafile', help="Path to datafile")
    parser.add_argument('destination', help="Path to export destination")
    parser.add_argument("name", help="Name of generated file")
    parser.add_argument("format", help="Format of generated file")
    parser.add_argument('value', choices=['corrected_area', 'isotopologue_fraction', 'mean_enrichment'])

    parser.add_argument('-m', '--metabolite', default='all',
                        help="Metabolite(s) to plot. For all, type in 'all' ")
    parser.add_argument('-c', '--condition', default='all',
                        help="Condition(s) to plot. For all, type in 'all' ")
    parser.add_argument('-t', '--time', default='all',
                        help="Time(s) to plot. For all, type in 'all' ")
    parser.add_argument("-tp", "--template", default=0, action="store",
                        help="Path to template")

    logger.debug('Adding optionnal arguments for calling the plot methods')
    parser.add_argument('-sa', '--stacked_areaplot', action="store_true",
                        help='Create static stacked areaplot')
    parser.add_argument("-bp", "--barplot", action="store_true",
                        help='Create static barplot(for corrected_area and isotopologue_fraction)')
    parser.add_argument('-mb', '--meaned_barplot', action="store_true",
                        help='Create static barplot with meaned replicates(for corrected_area and '
                             'isotopologue_fraction)')
    parser.add_argument('-smp', '--static_mean_enrichment_plot', action="store_true",
                        help='Create static barplot with mean enrichment data')
    parser.add_argument('-smm', '--static_mean_enrichment_meanplot', action="store_true",
                        help='Create static barplot with mean enrichment data and meaned replicates')

    parser.add_argument('-IMP', '--interactive_mean_enrichment_plot', action="store_true",
                        help='Create interactive barplot with mean enrichment data')
    parser.add_argument('-IMM', '--interactive_mean_enrichment_meanplot', action="store_true",
                        help='Create interactive barplot with mean enrichment data and meaned replicates')
    parser.add_argument('-ISB', '--interactive_stacked_barplot', action="store_true",
                        help='Create interactive stacked barplot (for corrected area and isotopologue_fraction)')
    parser.add_argument('-ISM', '--interactive_stacked_meanplot', action="store_true",
                        help='Create interactive stacked barplot with meaned replicates (for corrected area and '
                             'isotopologue_fraction)')
    parser.add_argument('-ISA', '--interactive_stacked_areaplot', action="store_true",
                        help='Create interactive stacked areaplot')

    parser.add_argument('-hm', '--static_heatmap', action="store_true",
                        help='Create a static heatmap using mean enrichment data')
    parser.add_argument('-cm', '--static_clustermap', action="store_true",
                        help='Create a static heatmap with clustering using mean enrichment data')
    parser.add_argument('-HM', '--interactive_heatmap', action="store_true",
                        help='Create interactive heatmap using mean enrichment data')

    parser.add_argument('-s', '--stack', action="store_false",
                        help='Add option if barplots should be unstacked')
    parser.add_argument('-v', '--verbose', action="store_true",
                        help='Turns logger to debug mode')
    parser.add_argument('-a', '--annot', action='store_true',
                        help='Add option if annotations should be added on maps')

    return parser


def control_isoplot_plot(args, data_object, metabolite, conditions, times):
    """Function to control which plot methods are called depending on the
    arguments that were parsed"""

    logger.debug("Initiating plot object")

    static_plot = StaticPlot(args.stack, args.value, data_object.dfmerge,
                             args.name, metabolite, conditions, times, args.format, )

    interactive_plot = InteractivePlot(args.stack, args.value, data_object.dfmerge,
                                       args.name, metabolite, conditions, times)

    logger.debug("Selecting plot method")

    if args.stacked_areaplot:
        static_plot.stacked_areaplot()

    elif args.barplot:
        static_plot.barplot()

    elif args.meaned_barplot:
        static_plot.mean_barplot()

    elif args.static_mean_enrichment_plot:
        static_plot.mean_enrichment_plot()

    elif args.static_mean_enrichment_meanplot:
        static_plot.mean_enrichment_meanplot()

    elif args.interactive_mean_enrichment_plot:
        interactive_plot.mean_enrichment_plot()

    elif args.interactive_mean_enrichment_meanplot:
        interactive_plot.mean_enrichment_meanplot()

    elif args.interactive_stacked_barplot:
        interactive_plot.stacked_barplot()
        
    elif args.interactive_stacked_barplot and not args.stack:
        interactive_plot.unstacked_barplot()

    elif args.interactive_stacked_meanplot:
        interactive_plot.stacked_meanplot()

    elif args.interactive_stacked_meanplot and not args.stack:
        interactive_plot.unstacked_meanplot()

    elif args.interactive_stacked_areaplot:
        interactive_plot.stacked_areaplot()


def control_isoplot_map(args, data_object):
    """Function to control which map methods are called depending on the
    arguments that were parsed"""

    logger.debug("Initiating map object")

    mymap = Map(data_object.dfmerge, args.name, args.annot, args.format)

    logger.debug("Selecting plot method")

    if args.static_heatmap:
        mymap.build_heatmap()

    elif args.static_clustermap:
        mymap.build_clustermap()

    elif args.interactive_heatmap:
        mymap.interactive_heatmap()


def get_local_cli_input(name, data_object, logger):
    """
    Function to get input from user and check for errors in spelling.
    If an error is detected input is asked once more.
    This function is used for local implementation

    :param name: name of what we are looking for
    :type name: str
    :param data_object: IsoplotData object containing final clean dataframe
    :type data_object: class: 'isoplot.dataprep.IsoplotData'
    :param logger: Logger object
    :type logger: class: 'logging.Logger'
    :return: Desired string after parsing
    :rtype: list
    """

    if item == "all":
        desire = data_object.dfmerge[name].unique()
    else:
        is_error = True

        while is_error:
            try:
                # We show the list of possibilities to user
                print(data_object.dfmerge[name].unique())

                # Cli gives list of strings, se we must make words of them
                desire = [item for item in input("Input " + name
                                                 + "(s) to plot from list above (write 'all' for all to be plotted) "
                                                 ).split()]

                # Checking input for typos
                for item in desire:
                    if item == "all":
                        break
                    else:
                        assert item in data_object.dfmerge[name].unique()

            except AssertionError:
                logger.error(
                    "One or more of the chosen " + name + "(s) were not in list. Please check and try again.")

            except Exception as e:
                logger.error("Unexpected error: {}".format(e))

            else:
                logger.info("Chosen " + name + ": {}".format(desire))
                is_error = False

    return desire


def get_cli_input(arg, name, data_object, logger):
    """
    Function to get input from user and check for errors in spelling.
    If an error is detected input is asked once more.
    This function is used for galaxy implementation

    :param arg: list from which strings must be parsed
    :type arg: list
    :param name: name of what we are looking for
    :type name: str
    :param data_object: IsoplotData object containing final clean dataframe
    :type data_object: class: 'isoplot.dataprep.IsoplotData'
    :param logger: Logger object
    :type logger: class: 'logging.Logger'
    :return: Desired string after parsing
    :rtype: list

    """

    if item == "all":
        desire = data_object.dfmerge[name].unique()
    else:
        is_error = True

        while is_error:
            try:

                # Cli gives list of strings, se we must make words of them
                desire = [item for item in arg.split(",")]

                # Checking input for typos
                for item in desire:
                    if item == "all":
                        break
                    else:
                        assert item in data_object.dfmerge[name].unique()

            except AssertionError:
                logger.error(
                    "One or more of the chosen " + name +
                    "(s) were not in list. Please check and try again. Error: {}".format(item))

            except Exception as e:
                logger.error("Unexpected error: {}".format(e))

            else:
                logger.info("Chosen " + name + ": {}".format(desire))
                is_error = False

    return desire



from setuptools import setup, find_packages
import re

def open_readme_file():
    with open('README.rst') as f:
	     data = f.read()
    return data
	
# Version is maintained in the __init__.py file
with open("isoplot/__init__.py") as f:
    try:
        VERSION = re.findall(r"^__version__ = ['\"]([^'\"]*)['\"]", f.read(), re.M)[0]
    except IndexError:
        raise RuntimeError('Unable to determine version.')

setup(
	name = "isoplot",
	version = VERSION,
	description = 'Generate figures from Isocor output',
    classifiers = ["Development Status :: 4 - Beta",
                "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
                "Natural Language :: French",
                "Operating System :: OS Independent",
                "Programming Language :: Python :: 3.6",
                "Programming Language :: Python :: 3.7",
                "Programming Language :: Python :: 3.8",
                "Programming Language :: Python :: 3.9",
                "Topic :: Scientific/Engineering :: Bio-Informatics"],
    long_description = open_readme_file(),
	packages = find_packages(),
    install_requires = [
        "numpy>=1.19.1",
        "pandas>=1.1.1",
        "matplotlib>=3.3.1",
        "seaborn>=0.10.1",
        "natsort>=7.0.1",
        "bokeh==2.0.2",
		"ipywidgets>=7.5.1",
        "colorcet>=2.0.2",
        "openpyxl>=3.0.5",
        "xlrd>=1.2.0"
    ], 
	entry_points = {
        'console_scripts': [
            'isoplot = isoplot.isoplotcli:initialize_cli',
        ]},
    url = "https://forgemia.inra.fr/metatoul/isoplot2",
    author = "Loïc Le Grégam",
    author_email = "loic.le-gregam@insa-toulouse.fr"
    
)